/* Multipattern random searches using set Horspool's algorithm adapted to 
search inside a text compressed with (S,C)-Dense Code.
Lightweight Natural Language Text Compression: Information Retrieval 2006
Copyright (C) 2005 Antonio Faria.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author's contact: Antonio Faria, Dept. of Computer Science, University of
A Corua. Campus de Elvia s/n. Spain  fari@udc.es
*/

/*
BufferedReader is a modification of the buffering routines found in
"Nrgrep -- a fast and flexible pattern matching tool.
Copyright (C) 2000 Gonzalo Navarro"
http://www.dcc.uchile.cl/~gnavarro/software/sindex.html
*/


#include "BufferedReader.h"

/*------------------------------------------------------------------
 Creates a new buffer 
 ------------------------------------------------------------------*/
BufferRead bufCreate (unsigned int minDataInBuffer)
{
	BufferRead B = (BufferRead) malloc (sizeof(struct sBuffer));
	B->size = BUFFER_SIZE;
	B->data = (byte *) malloc (1+B->size+MAX_CODEWORD_SIZE);  //need extra space to break_pattern
	B->data++;
	B->dsize = 0;
	B->minDataInBuffer=minDataInBuffer;
	return B;
}

  
/*------------------------------------------------------------------
 Terminates use of buffer B 
 ------------------------------------------------------------------*/
void bufDestroy (BufferRead B)
{
	free (B->data-1);
	free (B);
} 
  
/*------------------------------------------------------------------
 Sets the file for buffer B 
------------------------------------------------------------------*/
void bufSetFile (BufferRead B, int file)
{
	B->file = file;
	B->fpos = 0;
	B->dsize = read (B->file,B->data,B->size);
}
  
/*------------------------------------------------------------------
 Gets buffer size of B 
------------------------------------------------------------------*/
int bufSize (BufferRead B)
{
	return B->size;
}

/*------------------------------------------------------------------
 Reads a new buffer starting at position next 
 ------------------------------------------------------------------*/
void bufLoad (BufferRead B, byte *pnext)
{ 
	register byte *src = pnext;
	register byte *dst = B->data;
	register int mov = B->dsize - (src - dst);

	/* move unused part to the beginning (memcpy is not portable) */
	B->dsize = mov;

	while (mov--) *dst++ = *src++;
	mov = B->size - B->dsize;
	B->dsize += read (B->file, dst, mov);
	B->fpos += mov;

	////fprintf(stderr,"\n  %ld bytes remain in  buffer ",B->dsize);
}
  
/*------------------------------------------------------------------
 Tells whether there is no more data in B + its file 
 ------------------------------------------------------------------*/
int bufEmpty (BufferRead B)
{
	return B->dsize == 0;
}

/*------------------------------------------------------------------
 Tells whether there is no more data to read in B's file 
------------------------------------------------------------------*/
int bufEof (BufferRead B)
{ 
	return B->dsize < B->size;
}

/*------------------------------------------------------------------
 Tells the file position of the current buffer 
------------------------------------------------------------------*/
int bufTextPos (BufferRead B)
{ 
	return B->fpos;
}

/*------------------------------------------------------------------
 Gives the area of memory of the current buffer 
 One text position before the given buffer can be touched 
------------------------------------------------------------------*/
void bufCurrent (BufferRead B, byte **base, byte **top)
{ 
	*base = B->data;
    if (!bufEof(B)){
    	*top = B->data + B->dsize - B->minDataInBuffer;
    }
	 else {
		*top = B->data + B->dsize;
	}

}



/*------------------------------------------------------------------
 ------------------------------------------------------------------*/
/*
EXAMPLE ....
int main(int argc, char* argv[])
{
int fIn; //file to read
BufferRead B;

		B = bufCreate(10);
		fIn = open ("in", O_RDONLY);
		if (fIn == -1)
			////fprintf(stderr,"Cannot read file %s","in");

		bufSetFile (B,fIn);

		 { byte *text,*top, *nextbuf,*pbeg,*pend;
			while ( !bufEmpty(B) ) {
				bufCurrent (B,&text,&top);
				pbeg = text; pend = top;

					while (pbeg <pend) {
						////fprintf(stderr,"[%c]",*(pbeg++));
					}

					if (bufEof(B)) nextbuf = top;
					else nextbuf = pbeg;

					bufLoad (B, nextbuf);
			}
		}
	printf("Read bytes %d\n", B->fpos-1);
	close (fIn);
	bufDestroy (B);

}
*/





//
///*-----------------------------------------------------------------------
// Arquivo    : Set horspool modified.h
// Function   : Implments a BufferReader keeping at least the "minDataInBuffer"
//  			  previous bytes read available to go back in the buffer read.
// Last change: 19/01/2005
// ------------------------------------------------------------------------*/
//
//
//	/* Buffer management */
//
//#include "BufferedReader.h"
//
//
//BufferRead bufCreate (unsigned int minDataInBuffer,unsigned int minDataInBufferRight)
//
//	/* Creates a new buffer */
//
//  { BufferRead B = (BufferRead) malloc (sizeof(struct sBuffer));
//    B->size = BUFFER_SIZE;
//    B->data = (byte *) malloc (1+ B->size +4);
//    B->data++;
//    B->dsize = 0;
//    B->minDataInBuffer=minDataInBuffer;
//    B->minDataInBufferRight = minDataInBufferRight;
//    fprintf(stderr,"\n CREADO BUFFER DE TAMAO %ld\n",BUFFER_SIZE);
//    return B;
//  }
//
//void bufDestroy (BufferRead B)
//
//	/* Terminates use of buffer B */
//
//  {
//  	 free (B->data-1);
//    free (B);
//  }
//
//
//void bufSetFile (BufferRead B, int file)
//
//	/* Sets the file for buffer B */
//
//  { B->file = file;
//    B->fpos = 0;
//    B->dsize = read (B->file,B->data,B->size);
//    ////fprintf(stderr,"\n");
//
//  }
//
//int bufSize (BufferRead B)
//
//	/* Gets buffer size of B */
//
//  { return B->size;
//  }
//
//void bufLoad (BufferRead B, byte *pnext)
//
//	/* Reads a new buffer starting at position next */
//
//  { register byte *src = pnext;
//    register byte *dst = B->data;
//    register int mov = B->dsize - (src - dst);
//
//	/* move unused part to the beginning (memcpy is not portable) */
//    B->dsize = mov;
//	//    fprintf(stderr,"\n BuffLoad: moviendo %ld bytes en buffer de lectura",mov);
//    while (mov--) *dst++ = *src++;
//    mov = B->size - B->dsize;
//    B->dsize += read (B->file, dst, mov);
//    //fprintf(stderr,"[[%c]]",*dst);
//    B->fpos += mov;   								//FUNCIONA OK, excepto para el resto del ltimo buffer utilizado.
//	 //    fprintf(stderr,"[[%ld]]",B->fpos);
//    //fprintf(stderr,"\n quedan %ld bytes en buffer de lectura",B->dsize);
//  }
//
//int bufEmpty (BufferRead B)
//
//  	/* Tells whether there is no more data in B + its file */
//
//  { return B->dsize == 0;
//  }
//
//int bufEof (BufferRead B)
//
//  	/* Tells whether there is no more data to read in B's file */
//
//  { return B->dsize < B->size;
//  }
//
//int bufTextPos (BufferRead B)
//
//	/* Tells the file position of the current buffer */
//
//  { return B->fpos;
//  }
//
//void bufCurrent (BufferRead B, byte **base, byte **top)
//
//	/* Gives the memory area of the current buffer */
////  {
////  	 *base = B->data;
////	 *top = B->data + B->dsize;
////  }
//  	
//  { *base = B->data;
//    if (!bufEof(B)){
//    		*top = B->data + B->dsize - B->minDataInBufferRight;
//		    //fprintf(stderr,"\n dsize = [%ld], top = [%ld]",B->dsize, *top - B->data);
//		    
//    }
//	 else {*top = B->data + B->dsize;
//		    //fprintf(stderr,"\n dsize = [%ld], top = [%ld]",B->dsize, *top - B->data);
//	 }
//
//  }
//	
//	
//void bufEndData (BufferRead B, byte **base) {
//		*base = B->data + (B->size);
//}
//	
//	/*
//
//int main(int argc, char* argv[])
//{
//int fIn; //file to read
//BufferRead B;
//unsigned long n,pos;
//unsigned int mindataInBufferRight,i;
//mindataInBufferRight = 3;
//
//		B = bufCreate(0,mindataInBufferRight);
//		fIn = open ("in", O_RDONLY);
//		if (fIn == -1) {
//			fprintf(stderr,"Cannot read file %s\n\n","in");
//			exit(0);
//		}
//
//		bufSetFile (B,fIn);
//
//		 { byte *text,*top, *nextbuf,*pbeg,*pend;
//			while ( !bufEmpty(B) ) {
//				bufCurrent (B,&text,&top);			//internamente mantiene "top <- Buffer.end - minDataInBufferRight"
//				pbeg = text; pend = top;
//				n= pend - pbeg;
//				pos = 0;
//					while ( pos <n )   {
//						fprintf(stderr,"[%c]",pbeg[pos]);
//						pos++;
//					}
//					
//					//muestra los bytes que quedan al final del buffer como "no accesibles"
//					fprintf(stderr,"   ");
//					for (i=0;i<mindataInBufferRight;i++) { fprintf(stderr,"[%c]",pbeg[n+i]); }
//					
//					pbeg = pbeg +pos - B->minDataInBuffer;
//					
//					if (bufEof(B)) nextbuf = top;  //necesario para que procese por completo ltimo buffer
//					else nextbuf = pbeg;			    //ntese que en general pbeg == pbeg+ pos - minDataInBuffer (aproximadamente top-minDataInBuffer - epsilon).
//															 //y el buffer nunca quedara vaco --> bucle infinito.
//					bufLoad (B, nextbuf);
//					fprintf(stderr,"\n");
//			}
//		}
//	fprintf(stderr,"Leidos %d\n", B->fpos-1);
//	close (fIn);
//	bufDestroy (B);
//
//}
//
//*/
